<?php

namespace Drupal\light_category\Plugin\Block;

use \Drupal\Core\Block\BlockBase;
use \Drupal\Core\Block\BlockPluginInterface;
use \Drupal\Core\Form\FormBuilderInterface;
use \Drupal\Core\Form\FormStateInterface;
use \Drupal\views\Views;
use \Drupal\taxonomy\Entity\Term;
use \Drupal\Core\Site\Settings;
use \Drupal\Component\Utility\Crypt;

/**
 * Provides a 'Light Category Block'.
 * 
 * @Block(
 * id = "light_category_block",
 * admin_label = @Translation("Light Category Block"),
 * category = @Translation("Views"),
 * )
 */
class LightCategoryBlock extends BlockBase implements BlockPluginInterface {

  /**
   * The light category trait
   */
  use \Drupal\light_category\Traits\LightCategoryTrait;
  use \Drupal\light_category\Traits\LightCategoryFormTrait;

  /**
   * {@inheritDoc}
   */
  public function defaultConfiguration() {
    $config =  $this->getDefaultConfiguration();
    $config['arguments']  = NULL;
    $config['pager-id']   = 0;
    $config['more-link']  = '';
    return $config;
}

  /**
   * {@inheritDoc}
   */
  public function getConfigurations(string $name) {
    if (!empty($this->configuration[$name])) return $this->configuration[$name];
    $config = \Drupal::config('light_category.settings')->get();
    return !empty($config[$name]) ? $config[$name] : $this->defaultConfiguration()[$name];
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $form = array_merge($this->getLightCategoryForm(), $form);

    $form['system']['#attributes']['class'][] = 'hidden';
    $form['arguments']['#disabled']           = FALSE;
    $form['views']['boolean-filter']['#ajax'] = FALSE;
    $form['views']['view-mode']['#ajax']      = FALSE;
    $form['views']['display']['#ajax']        = FALSE;

    if(!empty($form_state->getCompleteFormState()->getValue('settings')['views']['vocabulary'])) {
      $vocabulary = $form_state->getCompleteFormState()->getValue('settings')['views']['vocabulary'];
      if(!empty($vocabulary) || $vocabulary === 'all')  $form['arguments']['#selection_settings'] = []; 
      $form['arguments']['#selection_settings']['target_bundles'] = [$form_state->getCompleteFormState()->getValue('settings')['views']['vocabulary']];
    }

    $form['views']['pager-id'] = [
      '#type'           => 'number',
      '#title'          => $this->t('Pager ID'),
      '#description'    => $this->t("Unless you're experiencing problems with pagers related to this view, you should leave this at 0. If using multiple pagers on one page you may need to set this number to a higher value so as not to conflict within the ?page= array. Large values will add a lot of commas to your URLs, so avoid if possible."),
      '#default_value'  => $this->getConfigurations('pager-id'),
      '#min'            => 0,
      '#max'            => 9999
    ];

    $form['views']['more-link'] = [
      '#type'         => 'textfield',
      '#title'        => 'Custom link to page',
      '#default_value'=> empty($this->getConfigurations('more-link')) && $this->getConfigurations('term-alias') ? Term::load($this->getConfigurations('arguments'))->toUrl()->toString() : $this->getConfigurations('more-link'),
      '#states'           => [
        'invisible'       => [
          '#term-alias'   => ['checked' => TRUE]
        ],
      ],
    ];
    return $form;
  }

  /**
   * Ajax callback function
   *
   * @param array $form
   * @param FormStateInterface $form_state
   * @return void
   */
  public function lightCategoryCallback(array &$form, FormStateInterface $form_state) {
    $triger =  $form_state->getTriggeringElement()['#id'];
    $el     = $form_state->getValue('settings');

    switch($triger) {

      case 'view':
        $form['settings']['views']['display']['#options']        = $display = $this->getViewPluginBlock(Views::getView($el['views']['view'])->storage->get('display'));
        $form['settings']['views']['view-mode']['#options']      = array_merge(['none'=>'- None -'], $this->getSelect('viewMode', Views::getView($el['views']['view'])));
        $form['settings']['views']['boolean-filter']['#options'] = array_merge(['none'=>'- None -'], $this->getSelect('fieldBool', Views::getView($el['views']['view'])));
      break;

    }
    $form_state->setRebuild(TRUE);
    return $form['settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['view']            = $form_state->getValue('views')['view'];
    $this->configuration['display']         = $form_state->getValue('views')['display'];
    $this->configuration['view-mode']       = $form_state->getValue('views')['view-mode'];
    $this->configuration['vocabulary']      = $form_state->getValue('views')['vocabulary'];
    $this->configuration['boolean-filter']  = $form_state->getValue('views')['boolean-filter'];
    $this->configuration['pager-id']        = $form_state->getValue('views')['pager-id'];
    $this->configuration['more-link']       = $form_state->getValue('views')['more-link'];
    $this->configuration['arguments']       = $form_state->getValue('arguments');
    parent::blockSubmit($form, $form_state);
  }

  public function buildExclusion(string $action) {
    $build;
    switch ($action) {

      case 'hide':
        $build = [];
      break;

      case 'error':
        $build = [
          '#prefix' => '<div class="view-empty">',
          '#markup' => $this->t('Sorry, but it looks like you have deleted / empty the view / display of the view, or the arguments are incorrect.'),
          '#suffix' => '</div>',
        ];
      break;

    }
    return $build;
  }


  /**
   * {@inheritdoc}
   */
  public function build() {

    $view           = $this->configuration['view'];
    $display        = $this->configuration['display'];
    $arguments      = $this->configuration['arguments'];
    $booleanFilter  = $this->configuration['boolean-filter'];
    $viewMode       = $this->configuration['view-mode'];
    $pagerId        = $this->configuration['pager-id'];
    $moreLink       = $this->configuration['more-link'];
    /**
     * Init view
     */
    $view = Views::getView($view);
    if(empty($view) &&  !$view->access($display)) return $this->buildExclusion('error');
    $view->setDisplay($display);
    $view = $view->getDisplay();
    $view->view->setArguments([$arguments]);

    /**
     * Set view mode
     */
    if(!$view->view->display_handler->usesFields() && !empty($viewMode) && $viewMode != 'none') {
      $row_option  = $view->view->display_handler->getOption('row');
      $view->view->display_handler->overrideOption('row', $row_option['options']['view_mode']  = $viewMode);
    }

    /**
     * Set filter
     */
    if(!empty($booleanFilter) && $booleanFilter != 'none') {

      $booleanFilter = \Drupal::entityTypeManager()->getStorage('field_config')->load($this->configuration['boolean-filter'])->toArray();
      $view->view->addHandler($display, 'filter',  $booleanFilter['entity_type'] . '__' . $booleanFilter['field_name'], $booleanFilter['field_name'] . '_value', [
        'operator' => "=",
        'value' =>  "1",
        'plugin_id' =>  "boolean",
      ]);
    } 

    /**
     * Set more link
     */
    if(!empty($moreLink) && $view->view->display_handler->getOption('use_more')) {
      $view->view->display_handler->setOption('link_display', 'custom_url');
      $view->view->display_handler->setOption('link_url', $moreLink);
    }
    
    /**
     * Set pager id
     */
    if(!empty($pagerId && !empty($view->view->display_handler->getOption('pager')['options']['id']))) {
      $pager  = $view->view->display_handler->getOption('pager');
      $view->view->display_handler->overrideOption('pager', $pager['options']['id']  = $pagerId);
    }

    $view->preExecute();
    $view->execute();
    $view =  $view->buildRenderable();

    if($view['#view']->getDisplay()->options['block_hide_empty'] && empty($view['#view']->result)) return $this->buildExclusion('hide');

    $view['#view_id'] = $view['#view']->storage->id();
    $view['#view_display_show_admin_links'] = $view['#view']->getShowAdminLinks();
    $view['#view_display_plugin_id'] = $view['#view']->display_handler->getPluginId();
    views_add_contextual_links($view, 'block', $display);
    return  $view;
  }
}